﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Autodesk.Revit.DB;

namespace StairsAutomation
{
    class CurvedStairsRunConfiguration : IStairsRunConfiguration
    {

        public CurvedStairsRunConfiguration(int riserNumber, double bottomElevation, 
                                        double desiredTreadDepth, double width, 
                                        double innerRadius, Autodesk.Revit.Creation.Application appCreate)
        {
            m_riserNumber = riserNumber;
            m_bottomElevation = bottomElevation;
            m_innerRadius = innerRadius;
            m_outerRadius = innerRadius + width;
            m_incrementAngle = desiredTreadDepth / (m_innerRadius + width / 2.0);
            m_includedAngle = m_incrementAngle * (riserNumber - 1);

            m_center = new XYZ(0, 0, bottomElevation);
            m_appCreate = appCreate;
        }

        private int m_riserNumber;
        private double m_bottomElevation;
        private double m_innerRadius;
        private double m_outerRadius;
        private double m_incrementAngle;
        private double m_includedAngle;
        private XYZ m_center;
        private Autodesk.Revit.Creation.Application m_appCreate;

        #region IStairsRunConfiguration Members

        public double GetRunElevation()
        {
            return m_bottomElevation;
        }

        public IList<Autodesk.Revit.DB.Curve> GetRunStairsPath()
        {
            List<Curve> ret = new List<Curve>();
            Arc arc = m_appCreate.NewArc(m_center, (m_innerRadius + m_outerRadius) / 2.0, 0, m_includedAngle, XYZ.BasisX, XYZ.BasisY);
            ret.Add(arc);

            return ret;
        }

        public IList<Autodesk.Revit.DB.Curve> GetRunRiserCurves()
        {
            double incAngle = 0.0;
            XYZ center = XYZ.Zero;
            List<Curve> ret = new List<Curve>();

            for (int i = 0; i < m_riserNumber - 1; i++)
            {
                XYZ start = center + new XYZ(m_innerRadius * Math.Cos(incAngle), m_innerRadius * Math.Sin(incAngle), 0);
                XYZ end = center + new XYZ(m_outerRadius * Math.Cos(incAngle), m_outerRadius * Math.Sin(incAngle), 0);
                ret.Add(Line.get_Bound(start, end));

                incAngle += m_incrementAngle;
            }

            // last one handled manually to ensure that it intersects end curves
            IList<Curve> boundaryCurves = GetRunBoundaryCurves();

            ret.Add(Line.get_Bound(boundaryCurves[0].Evaluate(1.0, true), boundaryCurves[1].Evaluate(1.0, true)));

            return ret;
        }

        public IList<Autodesk.Revit.DB.Curve> GetRunBoundaryCurves()
        {
            List<Curve> ret = new List<Curve>();
            Arc arc = m_appCreate.NewArc(m_center, m_innerRadius, 0, m_includedAngle, XYZ.BasisX, XYZ.BasisY);
            ret.Add(arc);

            arc = m_appCreate.NewArc(m_center, m_outerRadius, 0, m_includedAngle, XYZ.BasisX, XYZ.BasisY);
            ret.Add(arc);

            return ret;
        }

        #endregion
    }
}
