﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Autodesk.Revit.DB;

namespace StairsAutomation
{
    public class StraightStairsRunConfiguration : IStairsRunConfiguration
    {
        public StraightStairsRunConfiguration(int riserNumber, double bottomElevation, double desiredTreadDepth, double width)
        {
            m_riserNumber = riserNumber;
            m_bottomElevation = bottomElevation;
            m_desiredTreadDepth = desiredTreadDepth;
            m_width = width;
            m_runOffset = new XYZ(0, (riserNumber - 1) * desiredTreadDepth, 0);
            m_widthOffset = new XYZ(m_width, 0, 0);
        }

        private int m_riserNumber;
        private double m_bottomElevation;
        private double m_desiredTreadDepth;
        private double m_width;
        private XYZ m_runOffset;
        private XYZ m_widthOffset;

        #region IStairsRunConfiguration Members

        public double GetRunElevation()
        {
            return m_bottomElevation;
        }

        public IList<Autodesk.Revit.DB.Curve> GetRunStairsPath()
        {
            XYZ start = new XYZ(m_width / 2.0, 0, m_bottomElevation);
            XYZ end = start + m_runOffset;
            Line curve1 = Line.get_Bound(start, end);

            List<Curve> ret = new List<Curve>();
            ret.Add(curve1);
            return ret;
        }

        public IList<Autodesk.Revit.DB.Curve> GetRunRiserCurves()
        {
            List<Curve> ret = new List<Curve>();

            for (int i = 0; i < m_riserNumber - 1; i++)
            {
                XYZ start = new XYZ(0, i * m_desiredTreadDepth, m_bottomElevation);
                XYZ end = start + m_widthOffset;

                ret.Add(Line.get_Bound(start, end));
            }

            // last one handled manually to ensure that it intersects end curves
            IList<Curve> boundaryCurves = GetRunBoundaryCurves();

            ret.Add(Line.get_Bound(boundaryCurves[0].Evaluate(1.0, true), boundaryCurves[1].Evaluate(1.0, true)));

            return ret;
        }

        public IList<Autodesk.Revit.DB.Curve> GetRunBoundaryCurves()
        {
            XYZ start = new XYZ(0, 0, m_bottomElevation);
            XYZ end = start + m_runOffset;
            Line curve1 = Line.get_Bound(start, end);

            List<Curve> ret = new List<Curve>();
            ret.Add(curve1);

            start = new XYZ(m_width, 0, m_bottomElevation);
            end = start + m_runOffset;
            Line curve2 = Line.get_Bound(start, end);
            ret.Add(curve2);

            return ret;
        }

        #endregion
    }
}
