﻿using System;
using System.Collections.Generic;
using Autodesk.Revit.DB;
using Autodesk.Revit.DB.Architecture;

namespace StairsAutomation
{
	/// <summary>
	/// Description of StairsBoundRun.
	/// </summary>
	public class StairsBoundRun : StairsStandardRun
	{
		public StairsBoundRun(Stairs stairs, Level bottomLevel, int numberOfLandings ):
			base(stairs, bottomLevel, numberOfLandings)
        {
            EqualizeRuns = true;
            
        }

		public StairsBoundRun(Stairs stairs, Level bottomLevel, int numberOfLandings, Transform transform):
			base(stairs, bottomLevel, numberOfLandings, transform)
        {
            EqualizeRuns = true;
            
        }
        
        public override IStairsLandingConfiguration GetLanding(Line riser1Line, Line riser2Line, XYZ runDirection, double runElevation)
        {
        	IStairsLandingConfiguration landingConfiguration = new StairsBoundedLandingConfiguration(riser1Line, riser2Line, runDirection, runElevation, GetBoundaryCurves(), AppCreate);
        	return landingConfiguration;
        }
        
        public void SetupRunOffset()
        {
        	double totalWidth = 2.0 * Length * Math.Cos(Math.PI/6.0);
        	RunOffset = totalWidth - 2.0 * RunWidth;
        }
        
        private double Length
        {
        	get
        	{
        		return 12.5;
        	}
        }
        
        public IList<Curve> GetBoundaryCurves()
        {
        	double length = Length;
        	// Hexagon starting at 0, 0
        	List<Curve> curves = new List<Curve>();
        	
        	XYZ startPoint = new XYZ(0, -2.5, 0);
        	XYZ previousPoint = new XYZ(startPoint.X, startPoint.Y, startPoint.Z);
        	double angle = 0;
        	for (int i = 0; i < 5; i ++)
        	{
        		XYZ nextPoint = previousPoint + new XYZ(length * Math.Sin(angle), length * Math.Cos(angle), 0);
        		curves.Add(Line.get_Bound(previousPoint, nextPoint));
        		
        		angle += Math.PI / 3.0;
        		previousPoint = nextPoint;
        	}
        	
        	// Last one, route back to first
        	curves.Add(Line.get_Bound(previousPoint, startPoint));
        	
        	return curves;
        }
	}
}
