﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using Autodesk.Revit.DB;
using Autodesk.Revit.DB.IFC;

namespace BIM.IFC.Export.UI
{
    /// <summary>
    /// Interaction logic for IFCExport.xaml
    /// </summary>
    public partial class IFCExport : Window
    {
        // The list of available configurations
        IFCExportConfigurationsMap m_configMap;

        /// <summary>
        /// The dialog result.
        /// </summary>
        IFCExportResult m_result = IFCExportResult.Invalid;

        public IFCExport(IFCExportConfigurationsMap configurationsMap, String selectedConfigName)
        {
            m_configMap = configurationsMap;

            InitializeComponent();

            UpdateCurrentSelectedSetupCombo(selectedConfigName);
       }

       private void UpdateCurrentSelectedSetupCombo(String selected)
       {
            // TODO: support additional user saved configurations.

            foreach (IFCExportConfiguration curr in m_configMap.Values)
            {
                currentSelectedSetup.Items.Add(curr.Name);  
            }
            if (selected == null || !currentSelectedSetup.Items.Contains(selected))
                currentSelectedSetup.SelectedIndex = 0;
            else
                currentSelectedSetup.SelectedItem = selected;
        }

        private void AddToConfigList(IFCExportConfiguration configuration)
        {
            m_configMap.Add(configuration);
        }

        public IFCExportResult Result
        {
            get { return m_result; }
        }

        public IFCExportConfigurationsMap GetModifiedConfigurations()
        {
            return m_configMap;
        }

        public IFCExportConfiguration GetSelectedConfiguration()
        {        
            String selectedConfigName = (String)currentSelectedSetup.SelectedItem;
            if (selectedConfigName == null)
                return null;
            
            return m_configMap[selectedConfigName];
        }

        public String GetSelectedConfigurationName()
        {
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration == null)
                return null;
            return configuration.Name;
        }

        public String GetFileExtension()
        {
            IFCExportConfiguration selectedConfig = GetSelectedConfiguration();
            IFCFileFormatAttributes selectedItem = new IFCFileFormatAttributes(selectedConfig.IFCFileType);
            return selectedItem.GetFileExtension();
        }

        public String GetFileFilter()
        {
            IFCExportConfiguration selectedConfig = GetSelectedConfiguration();
            IFCFileFormatAttributes selectedItem = new IFCFileFormatAttributes(selectedConfig.IFCFileType);
            return selectedItem.GetFileFilter();
        }

        private void buttonEditSetup_Click(object sender, RoutedEventArgs e)
        {
            IFCExportConfiguration selectedConfig = GetSelectedConfiguration();
            IFCExportConfigurationsMap configurationsMap = new IFCExportConfigurationsMap(m_configMap);
            IFCExporterUIWindow editorWindow = new IFCExporterUIWindow(configurationsMap, selectedConfig.Name);
            editorWindow.ShowDialog();
            if (editorWindow.DialogResult.HasValue && editorWindow.DialogResult.Value)
            {
                currentSelectedSetup.Items.Clear();
                m_configMap = configurationsMap;
                String selectedConfigName = editorWindow.GetSelectedConfigurationName();
                UpdateCurrentSelectedSetupCombo(selectedConfigName);
            }
        }

        private void buttonNext_Click(object sender, RoutedEventArgs e)
        {
            m_result = IFCExportResult.ExportAndSaveSettings;
            Close();
        }

        private void buttonCancel_Click(object sender, RoutedEventArgs e)
        {
            m_result = IFCExportResult.Cancel;
            Close();
        }

        private void buttonSaveSetAndClose_Click(object sender, RoutedEventArgs e)
        {
            m_result = IFCExportResult.SaveSettings;
            Close();
        }

        private void currentSelectedSetup_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            IFCExportConfiguration selectedConfig = GetSelectedConfiguration();

            if (selectedConfig != null)
            {
                // change description
                textBoxSetupDescription.Text = selectedConfig.Description;  
            }  
        }
    }
}
